package event

import (
	"context"

	"github.com/SamuelTariku/FortuneBet-Backend/internal/domain"
)

type Service interface {
	FetchLiveEvents(ctx context.Context) error
	FetchUpcomingEvents(ctx context.Context) error
	GetAllUpcomingEvents(ctx context.Context) ([]domain.UpcomingEvent, error)
	GetExpiredUpcomingEvents(ctx context.Context, filter domain.EventFilter) ([]domain.UpcomingEvent, error)
	GetPaginatedUpcomingEvents(ctx context.Context, filter domain.EventFilter) ([]domain.UpcomingEvent, int64, error)
	GetUpcomingEventByID(ctx context.Context, ID string) (domain.UpcomingEvent, error)
	// GetAndStoreMatchResult(ctx context.Context, eventID string) error
	UpdateFinalScore(ctx context.Context, eventID, fullScore string, status domain.EventStatus) error
	UpdateEventStatus(ctx context.Context, eventID string, status domain.EventStatus) error
}
